/*
     ______     __                     ___     __
    /\  _  \   /\ \                   /\_ \   /\_\
    \ \ \_\ \  \ \ \      __     ___  \//\ \  \/_/_
     \ \  __ \ `/__ \   /'__`\  / __`\  \ \ \   /\ \
      \ \ \/\ \/\\_\ \_/\ \_\ \/\ \_\ \_ \_\ \_ \_\ \_
       \ \_\ \_\ \_____\ \___\ \ \______\/\____\/\____\
        \/_/\/_/\/_____/\/___L\ \/______/\/____/\/____/
                          /\____/
                          \_/__/




                  An advanced gaming library

                 (c) Stephane APIOU 2000


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



*/

//! definition of all adgali features associated with ios
/*! \file
 * \ingroup ios
 * this file defines all adgali features associated with ios
 * and must be include when you use adgali
 * this file is included by agl.h
 */


#ifndef _adgali_ios_h
#define _adgali_ios_h

#include <stdio.h>

//! \name type of input output
//@{

#define IOS_NULL	0 //!< filename
#define IOS_FILE	1 //!< filename
#define IOS_STREAM	2 //!< stream
#define IOS_HANDLE	3 //!< handle
#define IOS_MEMORY	4 //!< memory

//@}

//! defines an io stream
/*! This structure defines io streams for adgali
 * all the members are private. Don't use them directly
 */
typedef struct agl_ios {
	int (*seek)(struct agl_ios *ios, int offset, int whence); //!< the seek function
	int (*read)(struct agl_ios *ios, void *buf, int size, int maxobj, void *supp); //!< the read function
	int (*write)(struct agl_ios *ios, void *buf, int size, int maxobj, void *supp); //!< the write function
	int (*close)(struct agl_ios *ios); //!< the close function
	union {
		FILE	*fstream; //!< file descriptor for a file stream
		int	handle; //!< handle for a file io
		struct {
			unsigned char *mem; //!< the memory zone
			unsigned long pos; //!< the position in this zone
			unsigned long size; //!< the size of this zone
		} mem; //!< members for memory ios
	} privates; //!< all these members are private
	char *filename; //!< the ( if defined ) name of the file
	int type; //!< type of input/output openened
} agl_ios; 

//! open a file with his filename and return an agl_ios struct pointing to it.
/*! this function open a file with his filename \a name and the mode \a type
 * and return when opened an agl_ios struct pointing to it
 * \param name The name of the file to open
 * \param type Specifies how to open the file see fopen for a description
 * \return the agl_ios struct or NULL if an error occurs.
 */

extern agl_ios *agl_ios_filename(char *name, char *type);

//! create an agl_ios struct pointing to a file described with a file stream.
/*! this function create an agl_ios structure pointing to a file which is
 * described with a stream io structure file
 * \param file the streaming structure pointer
 * \return the agl_ios struct or NULL if an error occurs.
 */

extern agl_ios *agl_ios_fstream(FILE *file);

//! create an agl_ios struct pointing to a file described with a file handle
/*! this function create an agl_ios structure pointing to a file which was
 * previously opened by an OPEN(1) call.
 * \param handle contains the handle of the file descriptor
 * \return the agl_ios struct or NULL if an error occurs.
 */

extern agl_ios *agl_ios_fhandle(int handle);

//! create an agl_ios struct pointing to a memory zone
/*! this function create an agl_ios structure pointing a memory zone which contains
 * the datas
 * \param mem pointer to the memory zone which contains the datas
 * \param size this is the size of the memory zone
 * \return the agl_ios struct or NULL if an error occurs.
 */

extern agl_ios *agl_ios_mem(void *mem, unsigned long size);

//! cerate an agl_ios struct pointing to a null decriptor
/*! this function create an agl_ios structure pointing to nothing 
 * \return the agl_ios struct
 */

extern agl_ios *agl_ios_null();

//! seek into the stream
/*! this function seek in a stream
 * \param ios an agl_ios pointer which defines the stream
 * \param offset an offset in then stream. The value could be positive or negative
 * \param whence indicates the type of offset. see also seek(2) function.
 * - SEEK_SET: force offset from the beginning of the stream
 * - SEEK_CUR: force offset from the current position of the stream
 * - SEEK_END: force offset from the end of the stream
 * \return the offset from the beginning of the stream
 * \note the position of the stream could not be negative and greater than the
 * stream size.
 */

#define agl_ios_seek(ios, offset, whence) (agl_assert(ios != NULL), \
	((ios)->seek(ios, offset, whence)))

//! tell the position in the stream
/*! this function tell the position of the read/write pointer in the stream
 * \param ios an agl_ios pointer which defines the stream
 * \return the offset from the beginning of the stream
 */

#define agl_ios_tell(ios) (agl_assert(ios != NULL), \
	((ios)->seek(ios, 0, SEEK_CUR)))

//! read the stream
/*! this function read the stream into a buffer
 * \param ios an agl_ios pointer which defines the stream
 * \param buf point to the read buffer. The size of this buffer should be big
 * enough to contain \a size * \a maxnum characters
 * \param size this is the size of one item
 * \param maxnum this the maximum number of item to read
 * \param supp this parameter is reserved for future use (socket for example)
 * \return the number of items read
 */

#define agl_ios_read(ios, buf, size, maxnum, supp) (agl_assert(ios != NULL), \
	((ios)->read(ios, buf, size, maxnum ,supp)))

//! write into the stream
/*! this function write a buffer into the stream
 * \param ios an agl_ios pointer which defines the stream
 * \param buf point to the write buffer. The size of this buffer should be big
 * enough to contain \a size * \a maxnum characters
 * \param size this is the size of one item
 * \param maxnum this the maximum number of item to read
 * \param supp this parameter is reserved for future use (socket for example)
 * \return the number of items written
 */

#define agl_ios_write(ios, buf, size, maxnum, supp) (agl_assert(ios != NULL), \
	((ios)->write(ios, buf, size, maxnum ,supp)))

//! close the stream
/*! this function close the stream
 * \param ios an agl_ios pointer which defines the stream
 * \return nothing
 */


#define agl_ios_close(ios) (agl_assert(ios != NULL), \
	((ios)->close(ios)))


#endif /* _agl_ios_h */
