/*
     ______     __                     ___     __
    /\  _  \   /\ \                   /\_ \   /\_\
    \ \ \_\ \  \ \ \      __     ___  \//\ \  \/_/_
     \ \  __ \ `/__ \   /'__`\  / __`\  \ \ \   /\ \
      \ \ \/\ \/\\_\ \_/\ \_\ \/\ \_\ \_ \_\ \_ \_\ \_
       \ \_\ \_\ \_____\ \___\ \ \______\/\____\/\____\
        \/_/\/_/\/_____/\/___L\ \/______/\/____/\/____/
                          /\____/
                          \_/__/




                  An advanced gaming library

                 (c) Stephane APIOU 2000


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



*/

//! definition of all adgali features associated with video functions
/*! \file
 * \ingroup video
 * this file defines all adgali features associated vith the video functions
 * and must be include when you use adgali
 * agl.h includes this file
 */


#ifndef _adgali__video_h
#define _adgali_video_h


//! \name pixel mask for the long representation. pixels stored in memory in RGBA order
//@{

#define AGL_RED		agl_swapbe32(0XFF000000) //!< green color mask
#define AGL_GREEN	agl_swapbe32(0X00FF0000) //!< green color mask
#define AGL_BLUE	agl_swapbe32(0X0000FF00) //!< blue color mask
#define AGL_ALPHA	agl_swapbe32(0X000000FF) //!< alpha mask

//@}

//! \name special alpha values
//@{

#define AGL_OPAQUE	0XFF //!< set alpha opaque
#define AGL_TRANSPARENT	0X00 //!< set alpha transparent


//@}

#define AGL_RGBA2LONG(value) agl_swapbe32(value) //!< transcode a long color to a long value

//! set red color
/*! this function sets the red color in a pixel value
 * \param value the color value is between 0 and 255 (0 is black)
 * \return the pixel value
 */

#define AGL_SETRED(value)	(agl_swapbe32(value<<24) & AGL_RED)

//! set green color
/*! this function sets the green color in a pixel value
 * \param value the color value is between 0 and 255 (0 is black)
 * \return the pixel value
 */

#define AGL_SETGREEN(value)	(agl_swapbe32(value << 16) & AGL_GREEN)

//! set blue color
/*! this function sets the blue color in a pixel value
 * \param value the color value is between 0 and 255 (0 is black)
 * \return the pixel value
 */

#define AGL_SETBLUE(value)	(agl_swapbe32(value << 8) & AGL_BLUE)

//! set alpha channel
/*! this function sets the alpha channel in a pixel value
 * \param value the alpha value or this pixel (255 is opaque, 0 is transparent)
 * \return the pixel value
 */

#define AGL_SETALPHA(value)	(agl_swapbe32(value) & AGL_ALPHA) //!< set alpha

//! \name video mode options
//@{

#define AGL_FULLSCREEN	1 //!< full screen mode

//@}

//! \name specify the type of channel separation or combination to use
//@{

#define AGL_RGB	0 //!< specify a RGB (Red, Green, Blue) combination or separation
#define AGL_CMY	1 //!< specify a CMY (Cyan, Magenta, Yellow) combination or separation
#define AGL_HSV 2 //!< specify an HSV (hue, Saturation, Value) combination or separation
#define AGL_AVG 3 //!< specify an average ( average of RGB channel ie luminance)

//@}


struct agl_zone;
struct agl_zone_list;

typedef struct {
	void	(*free)(struct agl_zone *, int);
	void	(*refresh)(struct agl_zone *);
	void	(*dereference)(struct agl_zone *);
} agl_function;

//! defines an position
/*! This structure defines a pixel position
 */

typedef struct {
	short x; //!< x position
	short y; //!< y position
} agl_pos;

//! defines a vector or a size
/*! This structure defines a vector or a size
 */

typedef struct {
	unsigned short width; //!< width size
	unsigned short height; //!< height size
} agl_size;


//! defines an rectangle
/*! This structure defines a pixel rectangle
 */

typedef struct {
	short x; //!< x position
	short y; //!< y position
	unsigned short width; //!< rectangle width
	unsigned short height; //!< rectangle height
} agl_rect;

typedef struct agl_rect_list {
	agl_rect	*rect;
	struct agl_rect_list *next;
} agl_rect_list;

//! defines an a channel pixmap
/*! This structure defines a map of pixels for one channel of a pixmap
 */

typedef struct {
	agl_size	size; //!< this rectangles defines the size of the channel
	unsigned char	*pixels; //!< contains all the pixels (a pixel is a 8 bit value)
	unsigned short row_size; //!< this is the size of a row (pixels) in the pixel table
} agl_channel;

//! defines an a pixmap
/*! This structure defines a map of pixels
 */

typedef struct {
	agl_size	size; //!< this rectangle defines the size of the pixmap
	unsigned long *pixels; //!< contains all the pixels (a pixel is a 32 bits value)
	unsigned short row_size; //!< the size of a row (pixels) in the pixel table
	unsigned char alpha; //!< defines the tranparency of the pixmap
} agl_pix;


typedef struct agl_zone {
	agl_pix		pix;
	char		visible;
	long		zl_nb;
	struct agl_zone_list	*lower;
	struct agl_zone_list	*upper;
	struct agl_zone_list	*uplast;
	long 	type;
	long 	flags;
	agl_function *function;
} agl_zone;

typedef struct agl_zone_list {
	unsigned long	nb;
	agl_zone	*zone;
	struct agl_zone_list *next;
} agl_zone_list;

typedef struct screen {
	agl_zone	zone;
	void	*sublayer;
	char	bpp;
} agl_screen;

typedef	struct {
	agl_zone	zone;
	unsigned long color_key;
	char *image_name;
	char *alpha_name;
} agl_image;

typedef struct {
	int	xo;
	int 	yo;
	int	wm;
	int	wx;
} character;


//! \name zone types already known
//@{

#define ZONE_NONE	0 //!< defines of zone that contains nothing
#define ZONE_IMAGE	1 //!< defines a zone that contains an image

//@}

//! \name zone flags known
//@{

#define ZONE_TILED	0 //!< defines that the zone sub elements are tiled. they are no overlap calculation
#define ZONE_OVERLAPED	1 //!< defines that the zone sub elements could be overlapped. the sub zones are classified from bottom to top
#define ZONE_CONTAINER	2 //!< defines that the zone contain an image of the sub elements. this could accelerate overlaped zone drawing

//@}
/* image manipulation */

/* Load an image from a data source */

extern agl_image *agl_loadimg(agl_ios *src, int freesrc);

#define agl_loadimg_filename(file)	agl_loadimg(agl_ios_filename(file, "rb"), 1)
#define agl_loadimg_fstream(file)	agl_loadimg(agl_ios_fstream(file), 0)
#define agl_loadimg_fhandle(handle)	agl_loadimg(agl_ios_fstream(handle), 0)
#define agl_loadimg_mem(adr, size)	agl_loadimg(agl_ios_mem(adr, size), 1)

/* other images routines */

extern agl_image *agl_createimage(int width, int height);
extern void agl_freeimage(void *ptr, int cascade);
extern void agl_dereferenceimage(void *ptr);


/* zone manipulation */


extern int agl_fillzone(agl_zone *pzone, int width, int height, int flags);
extern void agl_emptyzone(agl_zone *pzone);
extern agl_zone *agl_createzone(int width, int height, int flags);
extern void agl_freezone(agl_zone *zone, int cascade);
extern void agl_originzone(agl_zone *zone, int x, int y);


//! \name transformation types for pixtranscat and pixtransalpha
//@{

#define PIX_ID		0 //!< no transformation
#define PIX_SYM0	1 //!< operate a vertical mirror
#define PIX_SYM45	2 //!< operate a 45 degree diagonal mirror
#define PIX_SYM90	3 //!< operate an horizontal mirror
#define PIX_SYM135	4 //!< operate a 135 degree diagonal mirror
#define PIX_ROT90	5 //!< operate a 90 degree rotation
#define PIX_ROT180	6 //!< operate a 180 degree rotation
#define PIX_ROT270	7 //!< operate a -90 degree rotation

//@}

//! copy one zone into another
/*! copy one zone into another one with clipping of destination zone 
 *  	and source zone alpha of the destination zone is supposed to be opaque
 *	if psrcrect and pdstrect are NULL, no clipping is done
 *	alpha of original dest zone is 255 (opaque)
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

#define	agl_pixcat(psrc, pdst, psrcrect, pdstrect, pos)	agl_pixtranscat(psrc, pdst, psrcrect, pdstrect, pos, PIX_ID)

//! copy one zone into another with alpha calculation
/*! copy one zone into another one with clipping of destination zone 
   	and source zone and apply alpha to zones
	if psrcrect and pdstrect are NULL, this is a real alpha copy
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

#define	agl_pixalpha(psrc, pdst, psrcrect, pdstrect, pos)	agl_pixtransalpha(psrc, pdst, psrcrect, pdstrect, pos, PIX_ID)

//! copy one zone into another
/*! copy one zone into another one with clipping of destination zone 
 *  	and source zone alpha of the destination zone is supposed to be opaque
 *	if psrcrect and pdstrect are NULL, no clipping is done
 *	alpha of original dest zone is 255 (opaque)
 *	apply transformation to source before copying to dest 
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 *  \param type the type of copy.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

extern int agl_pixtranscat(agl_pix *psrc, agl_pix *pdst, agl_rect *psrcrect, agl_rect *pdstrect, agl_pos *pos, int type);

//! copy one zone into another with alpha calculation
/*! copy one zone into another one with clipping of destination zone 
   	and source zone and apply alpha to zones
	if psrcrect and pdstrect are NULL, this is a real alpha copy
 	apply transformation to source before copying to dest
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 *  \param type the type of copy.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

extern int agl_pixtransalpha(agl_pix *psrc, agl_pix *pdst, agl_rect *psrcrect, agl_rect *pdstrect, agl_pos *pos, int type);

extern int agl_pixfill(agl_pix *ptr, agl_rect *rect, unsigned long color);
extern int agl_pixset(agl_pix *ptr, agl_rect *rect, unsigned long mask, unsigned long color);
extern int agl_pixkey(agl_pix *ptr, agl_rect *rect, unsigned long color);

#endif /* _agl_video_h */
